@@ -1,9 +1,9 @@
 /*
  * Listener loop for subsystem library libss.a.
  *
- *	$Header: lib/ss/SCCS/s.listen.c 1.18 03/03/30 22:26:12-05:00 tytso@think.thunk.org $
- *	$Locker: <Not implemented> $
- * 
+ *	$Header$
+ *	$Locker$
+ *
  * Copyright 1987, 1988 by MIT Student Information Processing Board
  *
  * Permission to use, copy, modify, and distribute this software and
@@ -28,7 +28,7 @@ static ss_data *current_info;
 static jmp_buf listen_jmpb;
 static sigret_t (*sig_cont)(int);
 
-static sigret_t print_prompt(int sig)
+static sigret_t print_prompt(int sig __SS_ATTR((unused)))
 {
     if (current_info->redisplay)
 	    (*current_info->redisplay)();
@@ -38,7 +38,7 @@ static sigret_t print_prompt(int sig)
     }
 }
 
-static sigret_t listen_int_handler(int sig)
+static sigret_t listen_int_handler(int sig __SS_ATTR((unused)))
 {
     putc('\n', stdout);
     signal(SIGINT, listen_int_handler);
@@ -56,7 +56,7 @@ int ss_listen (int sci_idx)
     jmp_buf old_jmpb;
     ss_data *old_info = current_info;
     char *line;
-    
+
     current_info = info = ss_info(sci_idx);
     sig_cont = (sigret_t (*)(int)) 0;
     info->abort = 0;
@@ -81,7 +81,7 @@ int ss_listen (int sci_idx)
 			line = input;
 		else
 			line = NULL;
-	
+
 		input[BUFSIZ-1] = 0;
 	}
 	if (line == NULL) {
@@ -89,7 +89,7 @@ int ss_listen (int sci_idx)
 		(void) signal(SIGCONT, sig_cont);
 		goto egress;
 	}
-		
+
 	cp = strchr(line, '\n');
 	if (cp) {
 	    *cp = '\0';
@@ -130,10 +130,12 @@ void ss_abort_subsystem(int sci_idx, int code)
 {
     ss_info(sci_idx)->abort = 1;
     ss_info(sci_idx)->exit_status = code;
-    
+
 }
 
-void ss_quit(int argc, const char * const *argv, int sci_idx, pointer infop)
+void ss_quit(int argc __SS_ATTR((unused)),
+	     const char * const *argv __SS_ATTR((unused)),
+	     int sci_idx, pointer infop __SS_ATTR((unused)))
 {
     ss_abort_subsystem(sci_idx, 0);
 }
@@ -149,7 +151,7 @@ static char *cmd_generator(const char *text, int state)
 	static char const * const * name;
 	ss_request_entry *request;
 	char		*ret;
-	
+
 	if (state == 0) {
 		len = strlen(text);
 		rqtbl = current_info->rqt_tables;
@@ -185,7 +187,8 @@ static char *cmd_generator(const char *text, int state)
 	return 0;
 }
 
-char **ss_rl_completion(const char *text, int start, int end)
+char **ss_rl_completion(const char *text, int start,
+			int end __SS_ATTR((unused)))
 {
 	if ((start == 0) && current_info->rl_completion_matches)
 		return (*current_info->rl_completion_matches)
